CREATE or replace FUNCTION lib.ininterval(itag text, imoment timestamp with time zone) RETURNS boolean
    LANGUAGE plpgsql
    AS $$ 
begin 
	itag := lower(btrim(itag));

	   if itag = E'now' then
		return  (( now() - imoment ) < interval '1 sec');

	elsif itag = E'yesterday' then
		return  ( imoment::date = ( now() - interval '1 day' )::date);

	elsif itag = E'today' then
		return  ( imoment::date = now()::date);

	elsif itag = E'tomorrow' then
		return  ( imoment::date = ( now() + interval '1 day' )::date);

	elsif itag = E'last7days' then
		return ( imoment > date_trunc('day', now() - interval '7 days') and imoment < now());

	elsif itag = E'lastweek' then
		return ( imoment > date_trunc('week',now() - interval '1 week') and imoment < date_trunc('week',now()));

	elsif itag = E'last14days' then
		return ( imoment > date_trunc('day', now() - interval '4 days') and imoment < now());

	elsif itag = E'lasttwoweeks' then
		return ( imoment > date_trunc('week',now() - interval '2 weeks') and imoment < date_trunc('week',now()));

	elsif itag = E'last30days' then
		return ( imoment > date_trunc('day', now() - interval '30 days') and imoment < now());

	elsif itag = E'lastmonth' then
		return ( imoment > date_trunc('month',now() - interval '1 month') and imoment < date_trunc('month',now()));

	elsif itag = E'lastyear' then
		return ( imoment > date_trunc('year',now() - interval '1 year') and imoment < date_trunc('year',now()));

	elsif itag = E'inpast' then
		return ( now() > imoment );

	elsif itag = E'thisweek' then
		return ( imoment > date_trunc('week',now()) and imoment < date_trunc('week',now() + interval '1 week'));

	elsif itag = E'thismonth' then
		return ( imoment > date_trunc('month',now()) and imoment < date_trunc('month',now() + interval '1 month'));

	elsif itag = E'thisyear' then
		return ( imoment > date_trunc('year',now()) and imoment < date_trunc('year',now() + interval '1 year'));

	elsif itag = E'nextweek' then
		return ( imoment > date_trunc('week',now() + interval '1 week') and imoment < date_trunc('week',now() + interval '2 weeks'));

	elsif itag = E'next14days' then
		return ( imoment > date_trunc('day',now() + interval '1 day') and imoment < date_trunc('day',now() + interval '5 days'));

	elsif itag = E'nexttwoweeks' then
		return ( imoment > date_trunc('week',now() + interval '1 week') and imoment < date_trunc('week',now() + interval '3 weeks'));

	elsif itag = E'next30days' then
		return ( imoment > date_trunc('day',now() + interval '1 day') and imoment < date_trunc('day',now() + interval '31 days'));

	elsif itag = E'nextmonth' then
		return ( imoment > date_trunc('month',now() + interval '1 month') and imoment < date_trunc('month',now() + interval '2 month'));

	elsif itag = E'nextyear' then
		return ( imoment > date_trunc('year',now() + interval '1 year') and imoment < date_trunc('year',now() + interval '2 year'));

	elsif itag = E'infuture' then
		return ( now() < imoment );
	
	else	
		raise exception E'Invalid interval';
		return null;
	end if;

end;	
$$;
