

create or replace function public.table_template_record ( itpl text, idata jsonb ) returns record as $$
declare 
    r record;
begin
    create temp table _tmp as select null::text as __t; 
    for r in select * from public.templates_table where template = itpl order by colname loop
        execute format('alter table _tmp add column is not exists %s %s ', r.colname, regexp_replace(r.coltype,'^.*:(.*)$','\1'));
        execute format('update _tmp set %s = ($1 #>> ''%s'')::%s',r.colname,r.colpath,coalesce(r.coltype,'text')) using idata;
    end loop;
    alter table _tmp drop __t;
    select into r * from _tmp;
    drop table _tmp;
    return r;
end;
$$ language plpgsql;


create or replace function public.table_template_spec ( itpl text ) returns text as $$
    select 
        trim(
            string_agg(',', 
                colname||' '||regexp_replace(coltype,'^.*:(.*)$','\1')
            ), 
        ',') 
    from (
        select distinct * 
        from public.templates_table 
        where template = $1 
        order by colname
    ) q;
$$ language sql;


create or replace function public.table_template_json ( itpl text, idata jsonb ) returns jsonb as $$
declare
    res jsonb;
    r record;
    buf text;
    q text = '';
begin
    for r in select * from public.templates_table where template = itpl order by colname loop
        buf := idata #>> r.colpath;
        if buf notnull then
            q := q || format(',''%s'', ''%s''::%s',r.colname,buf,coalesce(r.coltype,'text'));
        end if;    
    end loop;
    if not q = '' then
        execute format('select json_strip_nulls(json_build_object(%s))::jsonb',ltrim(q,',')) into res;
    end if;
    return res;
end;
$$ language plpgsql;


create or replace function public.json_template(ipattern jsonb, idata jsonb) returns jsonb as $$
"""
    pattern format - json with string substitutions:     
        "#{key,key,index,key\,with\,commas...}"
    
    -- TODO: substitute exec script as @{...}

"""
import json, re

try:
    pattern = json.loads(ipattern)
    data = json.loads(idata)
    _re = re.compile(r'^#{(.*)}$')
except Exception as ex:
    plpy.execute("select core.log('object template','error','{0}')".format(str(ex).replace("'", "`")), 0)
    return None

def _find(path, data):
    """
        path format: key,key,index,key\,with\,commas...
    """
    try:
        return reduce(lambda d, key: d[int(key) if key.isdigit() else key], re.split(r'(?<!\\),',path), data)
    except:
        return None

def _process(obj, data):
    if isinstance(obj,dict):
        return {k:_process(v,data) for k,v in obj.iteritems()}
    elif isinstance(obj,list):
        return [_process(v,data) for v in obj]
    elif isinstance(obj,basestring):
        m = _re.match(obj)
        if m and m.lastindex > 0:
            return _find(m.group(m.lastindex),data)
        else:
            return obj
    else:
        return obj

_res = _process(pattern,data)

return json.dumps(_res)

$$ language plpythonu;


create or replace function object_template( itpl text, idoclass text, idoctype text, idata jsonb ) returns jsonb as $$
    select json_template(pattern,$4) from public.templates_object 
        where 
            template = $1 
            or template = $1||'@'||$2
            or template = $1||'@'||$2||':'||$3
        order by template desc
        limit 1
    ;
$$ immutable language sql;    


