--
--
--
CREATE OR REPLACE FUNCTION notify_subscribe(
    i_channel character varying,
    i_recipient character varying,
    i_passphrase character varying,
    i_timeout integer DEFAULT (-1))
  RETURNS text AS
$$
declare
	r record;
begin
	if ( i_timeout < 0 ) then
		i_timeout := coalesce(_i('DEFAULT_NOTIFY_TIMEOUT'), 60);
	end if;	
	select into r * from notify_subscribers where channel = i_channel and recipient = i_recipient;
	if not found then
		insert into notify_subscribers (channel,recipient,passphrase,timeout) values(i_channel,i_recipient,i_passphrase,i_timeout);
		return E'0:subscribed';
	else
		if r.passphrase = i_passphrase then
			if r.enable then
				return E'1:already';
			else	
				update notify_subscribers set enable = true, timeout = i_timeout  where channel = i_channel and recipient = i_recipient;
				return E'0:enabled';
			end if;
		else
			return E'1:noaccess';
		end if;
	end if;
end;
$$ language plpgsql;


--
--
--
CREATE OR REPLACE FUNCTION notify_unsubscribe(
    i_channel character varying,
    i_recipient character varying,
    i_passphrase character varying)
  RETURNS text AS
$$
declare
	r record;
begin
	select into r * from notify_subscribers where channel = i_channel and recipient = i_recipient;
	if not found then
		return E'1:nothing';
	else
		if r.passphrase = i_passphrase then
			if r.enable then
				update notify_subscribers set enable = false where channel = i_channel and recipient = i_recipient;
				return E'0:unsubscribed';
			else	
				return E'1:already';
			end if;
		else
			return E'1:noaccess';
		end if;
	end if;
end;
$$ language plpgsql;


--
--
--
CREATE OR REPLACE FUNCTION notify_push(
    character varying,
    character varying,
    character varying,
    text)
  RETURNS void AS
$$
	insert into notify_messages (channel,sender,head,body) values ($1,$2,$3,$4);
$$ language sql;


--
--
--
CREATE OR REPLACE FUNCTION notify_pop(
    i_channel character varying,
    i_recipient character varying,
    i_passphrase character varying,
    i_order character varying DEFAULT 'desc'::character varying,
    i_limit integer DEFAULT 50)
  RETURNS SETOF notify_messages AS
$$
declare
	r record;
begin
	if ( i_order !~* E'^(asc|desc)$' ) then i_order := E'desc'; end if;
	for r in execute 'select 
		id, moment, channel, sender, head, body 
	from 
		notify_messages join (
			select 
				messageid 
			from 
				notify_stack join notify_subscribers on 
					notify_stack.recipient = notify_subscribers.recipient and
					notify_stack.channel = notify_subscribers.channel
			where
				notify_subscribers.recipient = $1 and
				notify_subscribers.passphrase = $2 and
				($3 = E''*'' or notify_subscribers.channel = $3) 
		) q on notify_messages.id = q.messageid
	order by moment '||i_order||' limit $4'  using i_recipient, i_passphrase, i_channel, i_limit
	loop
		delete from notify_stack where recipient = i_recipient and channel = r.channel and messageid = r.id;
		return next r;
	end loop;
end;
$$ language plpgsql;

--
--
--
CREATE OR REPLACE FUNCTION notify_messages_insert_t()
  RETURNS trigger AS
$$
declare
	r record;
begin
	for r in select 
			channel,recipient 
		from (
			select
			notify_stack.recipient, 
			notify_stack.channel, 
			notify_subscribers.timeout,
			( now()-notify_messages.moment ) as age 
		from
			notify_stack join notify_messages on notify_messages.id = notify_stack.messageid join notify_subscribers on ( notify_subscribers.channel = notify_stack.channel and notify_subscribers.recipient = notify_stack.recipient )
		where timeout > 0
		) q	
		where 
			q.age >  (q.timeout || E' seconds')::interval
	loop
		update notify_subscribers set enable = false where recipient = r.recipient and channel = r.channel;
		delete from notify_stack where recipient = r.recipient and channel = r.channel;
	end loop;	
	insert 
		into notify_stack 
		select 
			notify_subscribers.recipient,notify_subscribers.channel, NEW.id as messageid 
		from
			notify_subscribers 
		where	
			notify_subscribers.enable and notify_subscribers.channel = NEW.channel;
	perform pg_notify(E'notify_'||NEW.channel,'');		
	return NEW;
end;
$$ language plpgsql;
