
CREATE OR REPLACE FUNCTION plate_list_get(
    IN i_plate text,
    IN i_from_ts timestamp without time zone DEFAULT '-infinity'::timestamp without time zone,
    IN i_to_ts timestamp without time zone DEFAULT 'infinity'::timestamp without time zone,
	IN i_distance integer default 0,
    IN i_image character varying DEFAULT 'none'::character varying)
  RETURNS TABLE(
	id 				uuid, 
	ctime 			timestamp with time zone, 
	post_id 		uuid, 
	post_name 		text,
	channel 		integer, 
	channel_name 	text, 
	media_id 		bigint, 
	number 			text,
	direction		integer,
	velocity		integer,
	geodata			varchar,
	about 			text, 
	image 			bytea) AS
$BODY$

declare
 regex varchar;
begin 

if i_from_ts isnull then i_from_ts := '-infinity'::timestamp without time zone; end if;
if i_to_ts isnull   then i_to_ts   := ' infinity'::timestamp without time zone; end if;

regex := i_plate;

if( i_plate ~ E'[^\.]\*' or i_plate ~ E'^\*' ) then
	regex := replace(i_plate,'*','.*');
end if;

if ( regex !~ E'^\^' ) then
	regex := E'^'||regex;
end if;	

if ( regex !~ E'\$$' ) then
	regex := regex||E'$';
end if;

if regex ~ E'[\[\]\^\$\\\.\\+\*\?]' then 
	return query
	select 
		 p.guid
		,p.ctime
		,p.post_id
		,s.name,
		,p.channel
		,c.decription
		,p.media_id
		,p.number
		,p.direction
		,p.velocity
		,lib.geoformat(p.geodata,'dms'),
		,p.comment
		,case when i_image='none' then null else plate_image_get(p.post_id,p.channel,p.media_id, i_image) end as image	
	from
		platear p, post s, channel c 
	where
		s.id = p.post_id and
		c.post = p.post_id and
		c.channel = p.channel and
		p.number ~ regex and
		i_from_ts <= platear.ctime and platear.ctime < i_to_ts; 
elsif i_distance > 0 then
	return query
	select 
		 p.guid
		,p.ctime
		,p.post_id
		,s.name,
		,p.channel
		,c.decription
		,p.media_id
		,p.number
		,p.direction
		,p.velocity
		,lib.geoformat(p.geodata,'dms'),
		,p.comment
		,case when i_image='none' then null else plate_image_get(p.post_id,p.channel,p.media_id, i_image) end as image	
	from
		platear p, post s, channel c 
	where
		s.id = p.post_id and
		c.post = p.post_id and
		c.channel = p.channel and
		similarity(p.number,i_plate) <= i_distance and
		i_from_ts <= p.ctime and p.ctime < i_to_ts; 
else
	return query
	select 
		 p.guid
		,p.ctime
		,p.post_id
		,s.name,
		,p.channel
		,c.decription
		,p.media_id
		,p.number
		,p.direction
		,p.velocity
		,lib.geoformat(p.geodata,'dms'),
		,p.comment
		,case when i_image='none' then null else plate_image_get(p.post_id,p.channel,p.media_id, i_image) end as image	
	from
		platear p, post s, channel c 
	where
		s.id = p.post_id and
		c.post = p.post_id and
		c.channel = p.channel and
		p.number = i_plate and
		i_from_ts <= p.ctime and p.ctime < i_to_ts; 
	
return;

end;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100
  ROWS 1000;

