CREATE or replace FUNCTION test.plateload(ipost uuid, ichannel integer, imagepath text, iblocksize bigint, icount bigint, iopt xml, OUT errno bigint, OUT errmsg text) RETURNS record
    LANGUAGE plpgsql
    AS $$
declare
	i bigint;
	k bigint;
	_id bigint;
	_mid bigint;
	jr json;
	st timestamptz := clock_timestamp();
begin

raise notice '[START] %',st;

k := 0;

for i in 1..icount loop

	for jr in ( select api_plate_add ( json(format('{ "@header":{"vendor":"VIT", "class":"vitml.edge.api", "version":"0.1", "object":"plate", "action":"add"},  
		"guid": "%s",
		"ctime": "%s",
		"post": "%s",
		"channel": "%s",
		"rect": {
			"left": "%s",
			"top": "%s",
			"right": "%s",
			"bottom": "%s"
		},
		"number": "%s",
		"country": "%s",
		"numtype": "%s",
		"direction": "%s",
		"geodata": {
			"latitude": "%s",
			"longitude": "%s"
		},	
		"velocity": "%s",
		"validity": "%s"
		}',
		ct.uuid_generate_v1(), --guid
		lib.ts2us(clock_timestamp()), --ctime
		ipost, --post
		ichannel, --channel
		rx, ry, rx+0.08, ry+0.07, --rect
		test.random_plate('242'), --number
		804, --country
		1, --numtype
		0, --direction
		50.4440, 30.4497, --geodata
		round(random() * 71) + 20, --velocity
		100 --validity
		)))
	from (select random()*0.5 as rx, random()*0.5 as ry, generate_series as i from generate_series(1,iblocksize)) q) loop

		if not (jr ->> 'errcode') = '0' then 
			raise notice 'E: %', jr;
			continue;
		end if;	

		_id  = (jr ->> 'id')::bigint;
		_mid = (jr ->> 'scrollid')::bigint;

		perform plate_image_set(ipost,ichannel,_mid,'frame',true,lib.bytea_load(imagepath));

		update platear set media_id = _mid where id = _id;
			
	end loop;
	
	k := k + 1;

	raise notice '[%] plateload block loop: % in: %', clock_timestamp(),k, clock_timestamp()-st;

end loop;

errno := k*iblocksize;
errmsg := 'Ok';
raise notice '[END] [%] in: %', clock_timestamp(), clock_timestamp()-st;

return;

end;
$$;

CREATE OR REPLACE FUNCTION test.plateload1(IN ipost uuid, IN ichannel integer, IN imagepath text, IN iblocksize bigint, IN icount bigint, IN iopt xml, OUT errno bigint, OUT errmsg text)
  RETURNS record 
  LANGUAGE plpgsql
  AS
$BODY$
declare
	i bigint;
	k bigint;
	_id bigint;
	_mid bigint;
	jr json;
	st timestamptz := clock_timestamp();
begin

raise notice '[START] %',st;

k := 0;

for i in 1..icount loop

	for jr in ( select api_plate_add ( json(format('{ "@header":{"vendor":"VIT", "class":"vitml.edge.api", "version":"0.1", "object":"plate", "action":"add"},  
		"guid": "%s",
		"ctime": "%s",
		"post": "%s",
		"channel": "%s",
		"rect": {
			"left": "%s",
			"top": "%s",
			"right": "%s",
			"bottom": "%s"
		},
		"number": "%s",
		"country": "%s",
		"numtype": "%s",
		"direction": "%s",
		"geodata": {
			"latitude": "%s",
			"longitude": "%s"
		},	
		"velocity": "%s",
		"validity": "%s",
		"lane": "%s"
		}',
		ct.uuid_generate_v1(), --guid
		1465890773520817, --ctime
		ipost, --post
		ichannel, --channel
		0.869792, 0.932, 0.936632, 0.949714, --rect
		'<F2>ВВ4105АА', --number
		804, --country
		1, --numtype
		11, --direction
		50.453931, 30.449731, --geodata
		42, --velocity
		80, --validity
		2
		)))
	from (select random()*0.5 as rx, random()*0.5 as ry, generate_series as i from generate_series(1,iblocksize)) q) loop

		if not (jr ->> 'errcode') = '0' then 
			raise notice 'E: %', jr;
			continue;
		end if;	

		_id  = (jr ->> 'id')::bigint;
		_mid = (jr ->> 'scrollid')::bigint;

		perform plate_image_set(ipost,ichannel,_mid,'frame',true,lib.bytea_load(imagepath));

		update platear set media_id = _mid where id = _id;
			
	end loop;
	
	k := k + 1;

	raise notice '[%] plateload block loop: % in: %', clock_timestamp(),k, clock_timestamp()-st;

end loop;

errno := k*iblocksize;
errmsg := 'Ok';
raise notice '[END] [%] in: %', clock_timestamp(), clock_timestamp()-st;

return;

end;
$BODY$;

CREATE or replace FUNCTION test.plateload_manual(ipost uuid, ichannel integer, imagepath text, iblocksize bigint, icount bigint, iopt xml, OUT errno bigint, OUT errmsg text) RETURNS record
    LANGUAGE plpgsql
    AS $$
declare
	i bigint;
	k bigint;
	_id bigint;
	_mid bigint;
	jr json;
	st timestamptz := clock_timestamp();
begin

raise notice '[START] %',st;

k := 0;

for i in 1..icount loop

	for jr in ( select api_plate_add ( json(format('{ "@header":{"vendor":"VIT", "class":"vitml.edge.api", "version":"0.1", "object":"plate", "action":"add"},  
		"guid": "%s",
		"ctime": "%s",
		"post": "%s",
		"channel": "%s",
		"rect": {
			"left": "%s",
			"top": "%s",
			"right": "%s",
			"bottom": "%s"
		},
		"number": "%s",
		"country": "%s",
		"numtype": "%s",
		"direction": "%s",
		"geodata": {
			"latitude": "%s",
			"longitude": "%s"
		},	
		"velocity": "%s",
		"validity": "%s"
		}',
		ct.uuid_generate_v1(), --guid
		lib.ts2us(clock_timestamp()), --ctime
		ipost, --post
		ichannel, --channel
		rx, ry, rx+0.08, ry+0.07, --rect
		test.random_plate('242'), --number
		804, --country
		1, --numtype
		0, --direction
		50.4440, 30.4497, --geodata
		round(random() * 71) + 20, --velocity
		100 --validity
		)))
	from (select random()*0.5 as rx, random()*0.5 as ry, generate_series as i from generate_series(1,iblocksize)) q) loop

		if not (jr ->> 'errcode') = '0' then 
			raise notice 'E: %', jr;
			continue;
		end if;	

		_id  = (jr ->> 'id')::bigint;
		_mid = (jr ->> 'scrollid')::bigint;

		perform plate_image_set(ipost,ichannel,_mid,'frame',true,lib.bytea_load(imagepath));

	end loop;
	
	k := k + 1;

	raise notice '[%] plateload block loop: % in: %', clock_timestamp(),k, clock_timestamp()-st;

end loop;

errno := k*iblocksize;
errmsg := 'Ok';
raise notice '[END] [%] in: %', clock_timestamp(), clock_timestamp()-st;

return;

end;
$$;


